﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DEVSsharp;

namespace NebeskaTela
{
    class NebeskoTelo : Atomic
    {
        public Vektor centar;
        public Vektor pozicija;
        public double r;
        public double tRev;
        public double eps;
        public bool fiksno;

        public InputPort inputPortCentar;
        public OutputPort outputPortPozicija;

        List<PortValue> bafer;

        public NebeskoTelo(string name, TimeUnit tu, Vektor centar, Vektor pozicija, double r, double tRev, double eps, bool fiksno) : base(name, tu)
        {
            this.centar = centar;
            this.pozicija = pozicija;
            this.r = r;
            this.tRev = tRev;
            this.eps = eps;
            this.fiksno = fiksno;

            inputPortCentar = AddIP("inputPortCentar");
            outputPortPozicija = AddOP("outputPortPozicija");
        }
        public override void init()
        {
            bafer = new List<PortValue>();
        }

        public override bool delta_x(PortValue x)
        {
            if (x.port == inputPortCentar)
            {
                NebeskoTeloPoruka p = (NebeskoTeloPoruka)x.value;

                Vektor v = p.pozicija;

                Vektor pomeraj = v - centar;
                pozicija = pozicija + pomeraj;
                centar = v;
                pozicija.RotateAroundPoint(centar, 360.0 / tRev * TimeElapsed);

                bafer.Add(new PortValue(outputPortPozicija, new NebeskoTeloPoruka(pozicija, Name, r)));

                return true;
            }

            return false;
        }

        public override void delta_y(ref PortValue y)
        {
            if (bafer.Count > 0)
            {
                y = bafer[0];
                bafer.RemoveAt(0);
                return;
            }

            pozicija.RotateAroundPoint(centar, eps);
            bafer.Add(new PortValue(outputPortPozicija, new NebeskoTeloPoruka(pozicija, Name, r)));
        }

        public override double tau()
        {
            if (bafer.Count > 0)
            {
                return 0.0;
            }

            if (fiksno)
            {
                return double.MaxValue;
            }
            else
            {
                return tRev * eps / 360.0;
            }
        }

        public override string Get_s()
        {
            return this.Name + " " + pozicija;
        }
    }
}
