﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DEVSsharp;

namespace NebeskaTela
{
    class Sistem : Coupled
    {
        public InputPort inputPortKometaPoruka;

        public Sistem(string name) : base(name)
        {
            Vektor vektorSunca = new Vektor(0, 0);
            Vektor vektorZemlje = new Vektor(1080, 0);
            Vektor vektorMeseca = new Vektor(1100, 0);
            double rSunca = 100.0;
            double rZemlje = 10.0;
            double rMeseca = 3.0;

            NebeskoTeloPoruka suncePoruka = new NebeskoTeloPoruka(vektorSunca, "Sunce", rSunca);
            NebeskoTeloPoruka zemljaPoruka = new NebeskoTeloPoruka(vektorZemlje, "Zemlja", rZemlje);
            NebeskoTeloPoruka mesecPoruka = new NebeskoTeloPoruka(vektorMeseca, "Mesec", rMeseca);


            NebeskoTelo sunce = new NebeskoTelo("Sunce", TimeUnit.Sec, new Vektor(0, 0), vektorSunca, rSunca, 0.0, 0.0, true);
            NebeskoTelo zemlja = new NebeskoTelo("Zemlja", TimeUnit.Sec, vektorSunca, vektorZemlje, rZemlje, 365.0, 0.5, false);
            NebeskoTelo mesec = new NebeskoTelo("Mesec", TimeUnit.Sec, vektorZemlje, vektorMeseca, rMeseca, 30.0, 0.5, false);
            Stampac stampac = new Stampac("stampac", TimeUnit.Sec, suncePoruka, zemljaPoruka, mesecPoruka);

            AddModel(sunce);
            AddModel(zemlja);
            AddModel(mesec);
            AddModel(stampac);

            inputPortKometaPoruka = AddIP("inputPortKometaPoruka");


            double sunce_zemlja = Math.Sqrt((vektorSunca.X - vektorZemlje.X) * (vektorSunca.X - vektorZemlje.X) + (vektorSunca.Y - vektorZemlje.Y) * (vektorSunca.Y - vektorZemlje.Y));
            double zemlja_mesec = Math.Sqrt((vektorZemlje.X - vektorMeseca.X) * (vektorZemlje.X - vektorMeseca.X) + (vektorZemlje.Y - vektorMeseca.Y) * (vektorZemlje.Y - vektorMeseca.Y));
            double h = sunce_zemlja + zemlja_mesec + rMeseca;

            Vektor topLeft = new Vektor(vektorSunca.X - h, vektorSunca.Y + h);
            Vektor topRight = new Vektor(vektorSunca.X + h, vektorSunca.Y + h);
            Vektor bottomRight = new Vektor(vektorSunca.X + h, vektorSunca.Y - h);
            Vektor bottomLeft = new Vektor(vektorSunca.X - h, vektorSunca.Y - h);

            SuncevSistem suncevSistem = new SuncevSistem("Suncev Sistem", TimeUnit.Sec, topRight, topLeft, bottomLeft, bottomRight, 1.0, suncePoruka, zemljaPoruka, mesecPoruka);

            AddModel(suncevSistem);

            AddCP(sunce.outputPortPozicija, stampac.inputPortPozicija);
            AddCP(zemlja.outputPortPozicija, stampac.inputPortPozicija);
            AddCP(mesec.outputPortPozicija, stampac.inputPortPozicija);

            AddCP(zemlja.outputPortPozicija, mesec.inputPortCentar);

            AddCP(sunce.outputPortPozicija, suncevSistem.inputPortNebeskoTelo);
            AddCP(zemlja.outputPortPozicija, suncevSistem.inputPortNebeskoTelo);
            AddCP(mesec.outputPortPozicija, suncevSistem.inputPortNebeskoTelo);

            AddCP(inputPortKometaPoruka, suncevSistem.inputPortKometaPoruka);
            AddCP(suncevSistem.outputPortKometaPoruka, stampac.inputPortKometaPoruka);
            AddCP(suncevSistem.outputPortKometaUdarac, stampac.inputPortKometaUdarac);

            PrintCouplings();
        }
    }
}
