﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DEVSsharp;

namespace NebeskaTela
{
    class SuncevSistem : Atomic
    {
        public InputPort inputPortKometaPoruka;
        public InputPort inputPortNebeskoTelo;
        public OutputPort outputPortKometaPoruka;
        public OutputPort outputPortKometaUdarac;

        public Vektor topRight, topLeft, bottomLeft, bottomRight;
        double eps;

        NebeskoTeloPoruka sunce, zemlja, mesec;

        //public KometaPoruka kometa;

        List<PortValue> bafer;
        List<KometaPoruka> komete;
        List<KometaPoruka> zaBrisanje;

        public SuncevSistem(string name, TimeUnit tu, Vektor tr, Vektor tl, Vektor bl, Vektor br, double eps, NebeskoTeloPoruka s, NebeskoTeloPoruka z, NebeskoTeloPoruka m) : base(name, tu)
        {
            this.topLeft = tl;
            this.topRight = tr;
            this.bottomLeft = bl;
            this.bottomRight = br;
            this.eps = eps;
            this.sunce = s;
            this.zemlja = z;
            this.mesec = m;

            inputPortNebeskoTelo = AddIP("inputPortNebeskoTelo");
            inputPortKometaPoruka = AddIP("inputPortKometaPoruka");
            outputPortKometaPoruka = AddOP("outputPortKometaPoruka");
            outputPortKometaUdarac = AddOP("outputPortKometaUdarac");
        }

        public override void init()
        {
            bafer = new List<PortValue>();
            komete = new List<KometaPoruka>();
            zaBrisanje = new List<KometaPoruka>();
            //kometa = null;
        }

        public override bool delta_x(PortValue x)
        {
            if (x.port == inputPortKometaPoruka)
            {
                KometaPoruka tmp = (KometaPoruka)x.value;
                //kometa = (KometaPoruka)x.value;
                //komete.Add((KometaPoruka)x.value);
                komete.Add(tmp);
                bafer.Add(new PortValue(outputPortKometaPoruka, tmp));
                return true;
            }
            if(x.port == inputPortNebeskoTelo)
            {
                NebeskoTeloPoruka ntp = (NebeskoTeloPoruka)x.value;
                if (ntp.name.Equals("Sunce"))
                    sunce = ntp;
                else if (ntp.name.Equals("Zemlja"))
                    zemlja = ntp;
                else if (ntp.name.Equals("Mesec"))
                    mesec = ntp;

                foreach(var kometa in komete)
                {
                    double xx = kometa.kometa.X;
                    double yy = kometa.kometa.Y;

                    if (Math.Sqrt((xx - sunce.pozicija.X) * (xx - sunce.pozicija.X) + (yy - sunce.pozicija.Y) * (yy - sunce.pozicija.Y)) < sunce.r)
                    {
                        bafer.Add(new PortValue(outputPortKometaUdarac, new KometaPorukaUdarac(kometa.name, sunce.name)));
                        zaBrisanje.Add(kometa);
                    }
                    else if (Math.Sqrt((xx - zemlja.pozicija.X) * (xx - zemlja.pozicija.X) + (yy - zemlja.pozicija.Y) * (yy - zemlja.pozicija.Y)) < zemlja.r)
                    {
                        bafer.Add(new PortValue(outputPortKometaUdarac, new KometaPorukaUdarac(kometa.name, zemlja.name)));
                        zaBrisanje.Add(kometa);
                    }
                    else if (Math.Sqrt((xx - mesec.pozicija.X) * (xx - mesec.pozicija.X) + (yy - mesec.pozicija.Y) * (yy - mesec.pozicija.Y)) < mesec.r)
                    {
                        bafer.Add(new PortValue(outputPortKometaUdarac, new KometaPorukaUdarac(kometa.name, mesec.name)));
                        zaBrisanje.Add(kometa);
                    }
                }

                while (zaBrisanje.Count > 0)
                {
                    komete.Remove(zaBrisanje[0]);
                    zaBrisanje.RemoveAt(0);
                }

                return true;
            }

            return false;
        }

        public override void delta_y(ref PortValue y)
        {
            if(bafer.Count > 0)
            {
                y = bafer[0];
                bafer.RemoveAt(0);
                return;
            }

            foreach (var kometa in komete)
            {
                kometa.kometa = kometa.kometa + kometa.brzina * TimeElapsed;
                double xx = kometa.kometa.X;
                double yy = kometa.kometa.Y;

                if (xx > topLeft.X && xx < bottomRight.X && yy < topLeft.Y && yy > bottomRight.Y)
                {
                    if (Math.Sqrt((xx - sunce.pozicija.X) * (xx - sunce.pozicija.X) + (yy - sunce.pozicija.Y) * (yy - sunce.pozicija.Y)) < sunce.r)
                    {
                        bafer.Add(new PortValue(outputPortKometaUdarac, new KometaPorukaUdarac(kometa.name, sunce.name)));
                        zaBrisanje.Add(kometa);
                        //kometa = null;
                    }
                    else if (Math.Sqrt((xx - zemlja.pozicija.X) * (xx - zemlja.pozicija.X) + (yy - zemlja.pozicija.Y) * (yy - zemlja.pozicija.Y)) < zemlja.r)
                    {
                        bafer.Add(new PortValue(outputPortKometaUdarac, new KometaPorukaUdarac(kometa.name, zemlja.name)));
                        zaBrisanje.Add(kometa);
                        //kometa = null;
                    }
                    else if (Math.Sqrt((xx - mesec.pozicija.X) * (xx - mesec.pozicija.X) + (yy - mesec.pozicija.Y) * (yy - mesec.pozicija.Y)) < mesec.r)
                    {
                        bafer.Add(new PortValue(outputPortKometaUdarac, new KometaPorukaUdarac(kometa.name, mesec.name)));
                        zaBrisanje.Add(kometa);
                        //kometa = null;
                    }
                    else bafer.Add(new PortValue(outputPortKometaPoruka, kometa));
                }
                else
                {
                    bafer.Add(new PortValue(outputPortKometaPoruka, null));
                    zaBrisanje.Add(kometa);
                    //kometa = null;
                }
            }

            
            while(zaBrisanje.Count > 0)
            {
                komete.Remove(zaBrisanje[0]);
                zaBrisanje.RemoveAt(0);
            }
        }


        public override double tau()
        {
            if (bafer.Count > 0)
                return 0.0;

            if (komete.Count == 0)
            {
                return double.MaxValue;
            }
            else
            {
                //return kometa.eps / kometa.brzina.Length();
                double min = double.MaxValue;
                foreach (var kometa in komete)
                {
                    if (kometa.eps / kometa.brzina.Length() < min)
                        min = kometa.eps / kometa.brzina.Length();
                }

                return min;
            }
        }

        public override string Get_s()
        {
            string str = "";
            //return "Kometa: " + kometa;
            foreach (var kometa in komete)
            {
                str += kometa + "\n";
            }

            return str;
        }
    }
}
